<?php

namespace Spatie\Backup\Tasks\Backup;

use Illuminate\Support\Collection;
use Spatie\Backup\BackupDestination\Backup;
use Spatie\Backup\BackupDestination\BackupDestination;
use Spatie\Backup\Events\BackupHasFailed;
use Spatie\Backup\Events\BackupWasSuccessful;
use Spatie\Backup\Events\BackupZipWasCreated;
use Spatie\Backup\Exceptions\InvalidBackupJob;
use Spatie\Backup\Helpers\Format;
use Spatie\DbDumper\DbDumper;
use Exception;

class BackupJob
{
    /**  @var \Spatie\Backup\Tasks\Backup\FileSelection */
    protected $fileSelection;

    /** @var \Illuminate\Support\Collection */
    protected $dbDumpers;

    /** @var \Illuminate\Support\Collection */
    protected $backupDestinations;

    /** @var \Spatie\Backup\Tasks\Backup\TemporaryDirectory */
    protected $temporaryDirectory;

    public function __construct()
    {
        $this->doNotBackupFilesystem();
        $this->doNotBackupDatabases();

        $this->backupDestinations = new Collection();
    }

    /**
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     */
    public function doNotBackupFilesystem()
    {
        $this->fileSelection = FileSelectionFactory::noFiles();

        return $this;
    }

    /**
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     */
    public function doNotBackupDatabases()
    {
        $this->dbDumpers = new Collection();

        return $this;
    }

    /**
     * @param \Spatie\Backup\Tasks\Backup\FileSelection $fileSelection
     *
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     */
    public function setFileSelection(FileSelection $fileSelection)
    {
        $this->fileSelection = $fileSelection;

        return $this;
    }

    /**
     * @param array $dbDumpers
     *
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     */
    public function setDbDumpers(array $dbDumpers)
    {
        $this->dbDumpers = Collection::make($dbDumpers);

        return $this;
    }

    /**
     * @param string $diskName
     *
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     *
     * @throws \Spatie\Backup\Exceptions\InvalidBackupJob
     */
    public function backupOnlyTo($diskName)
    {
        $this->backupDestinations = $this->backupDestinations->filter(function (BackupDestination $backupDestination) use ($diskName) {
           return $backupDestination->getDiskName() === $diskName;
        });

        if (!count($this->backupDestinations)) {
            throw InvalidBackupJob::destinationDoesNotExist($diskName);
        }

        return $this;
    }

    /**
     * @param \Illuminate\Support\Collection $backupDestinations
     *
     * @return \Spatie\Backup\Tasks\Backup\BackupJob
     */
    public function setBackupDestinations(Collection $backupDestinations)
    {
        $this->backupDestinations = $backupDestinations;

        return $this;
    }

    public function run()
    {
        try {
            if (!count($this->backupDestinations)) {
                throw InvalidBackupJob::noDestinationsSpecified();
            }

            $this->temporaryDirectory = TemporaryDirectory::create();

            $zip = $this->createZipContainingAllFilesToBeBackedUp();

            $this->copyToBackupDestinations($zip);

            $this->temporaryDirectory->delete();
        } catch (Exception $exception) {
            consoleOutput()->error("Backup failed because {$exception->getMessage()}.");

            event(new BackupHasFailed($exception));
        }
    }

    /**
     * @return \Spatie\Backup\Tasks\Backup\Zip
     */
    protected function createZipContainingAllFilesToBeBackedUp()
    {
        $zip = Zip::create($this->temporaryDirectory->getPath(date('Y-m-d-His').'.zip'));

        $this->addDatabaseDumpsToZip($zip);

        $this->addSelectedFilesToZip($zip);

        event(new BackupZipWasCreated($zip));

        return $zip;
    }

    /**
     * @param \Spatie\Backup\Tasks\Backup\Zip $zip
     */
    protected function addSelectedFilesToZip(Zip $zip)
    {
        consoleOutput()->info('Determining files to backup...');

        $files = $this->fileSelection->getSelectedFiles();

        consoleOutput()->info("Zipping {$files->count()} files...");

        $zip->add($files->toArray());
    }

    /**
     * @param \Spatie\Backup\Tasks\Backup\Zip $zip
     */
    protected function addDatabaseDumpsToZip(Zip $zip)
    {
        $this->dbDumpers->each(function (DbDumper $dbDumper) use ($zip) {

            consoleOutput()->info("Dumping database {$dbDumper->getDbName()}...");

            $fileName = $dbDumper->getDbName().'.sql';
            $temporaryFile = $this->temporaryDirectory->getPath($fileName);
            $dbDumper->dumpToFile($temporaryFile);

            $zip->add($temporaryFile, $fileName);
        });
    }

    /**
     * @param \Spatie\Backup\Tasks\Backup\Zip $zip
     */
    protected function copyToBackupDestinations(Zip $zip)
    {
        $this->backupDestinations->each(function (BackupDestination $backupDestination) use ($zip) {

            try {
                if (!$backupDestination->isReachable()) {
                    throw new Exception("Could not connect to disk {$backupDestination->getDiskName()} because: {$backupDestination->getConnectionError()}");
                };

                $fileSize = Format::getHumanReadableSize($zip->getSize());

                $fileName = pathinfo($zip->getPath(), PATHINFO_BASENAME);

                consoleOutput()->info("Copying {$fileName} (size: {$fileSize}) to disk named {$backupDestination->getDiskName()}...");

                $backupDestination->write($zip->getPath());

                consoleOutput()->info("Successfully copied .zip file to disk named {$backupDestination->getDiskName()}.");

                event(new BackupWasSuccessful($backupDestination));
            } catch (Exception $exception) {
                consoleOutput()->error("Copying .zip file failed because: {$exception->getMessage()}.");

                event(new BackupHasFailed($exception));
            }
        });
    }
}
